import {Component, OnDestroy, OnInit} from '@angular/core';
import {LecturerDataService} from 'src/app/services/lecturer-data.service';
import {LecturerInfoService} from 'src/app/services/lecturer-info.service';
import {QuestionDao} from 'src/app/dao/question.dao';
import {AnswerSetDao} from 'src/app/dao/answer-set.dao';
import {GridOptions} from 'ag-grid-community';
import {ViewResponsesButtonComponent} from 'src/app/GridComponents/ViewResponsesButton/view-responses-button.component';
import {forkJoin} from 'rxjs';
import {Router} from '@angular/router';
import {QuestionAnswerPairDao} from 'src/app/dao/question-answer-pair.dao';

@Component({
  selector: 'app-lecturer-submissionsview',
  templateUrl: './lecturer-submissions-view.component.html',
  styleUrls: ['./lecturer-submissions-view.component.css']
})
export class LecturerSubmissionsViewComponent implements OnInit, OnDestroy {

  public lecturerId;
  public classCode;
  public paper;
  public questions: QuestionDao[];
  public answers: AnswerSetDao[];
  public questionAnswerPairs: QuestionAnswerPairDao[];
  public currentQuestion: QuestionDao;
  public rowData: any;
  public isLoading: Boolean;
  public gridOptions: GridOptions;
  public columnDefs = [
    {headerName: 'Index', field: 'index', sortable: true, lockPosition: true, width: 75},
    {headerName: 'Username', field: 'username', sortable: true, lockPosition: true, width: 150},
    {headerName: 'Submission Date', field: 'submissionDate', sortable: true, lockPosition: true, width: 150},
    {headerName: 'View Full Response', field: '', cellRendererFramework: ViewResponsesButtonComponent, lockPosition: true, width: 200},
  ];

  constructor(private lecturerDataService: LecturerDataService,
              private lecturerInfoService: LecturerInfoService,
              private router: Router) {
  }

  ngOnInit(): void {
    this.currentQuestion = undefined;
    this.isLoading = true;
    this.questions = [];
    this.answers = [];
    this.lecturerId = sessionStorage.getItem('username');
    this.classCode = this.lecturerDataService.classCode;
    this.paper = this.lecturerDataService.currentPaper;
    this.gridOptions = {
      context: {componentParent: this}
    };
    forkJoin([
      this.lecturerInfoService.getPaperAnswers(this.paper.paperId),
      this.lecturerInfoService.getPaperQuestions(this.paper.paperId)
    ]).subscribe((res) => {
      this.getPaperAnswers(res[0]);
      this.getPaperQuestions(res[1]);
      this.rowData = this.getRowData();
      this.goToResponse(this.answers[0]);
      this.isLoading = false;
    });
  }

  ngOnDestroy(): void {
    this.questions = [];
    this.answers = [];
    this.questionAnswerPairs = [];
  }

  private generatePairs(data: AnswerSetDao) {
    for (let i = 0; i < this.questions.length; i++) {
      if (this.questions[i] && data.answerSets[i]) {
        this.questionAnswerPairs.push(new QuestionAnswerPairDao(this.questions[i], data.answerSets[i]));
      }
    }
  }

  private getRowData(): any {
    const data = [];
    let index = 1;
    this.answers.forEach((a: AnswerSetDao) => {
      data.push({index: index, username: a.answerSets[0].username, submissionDate: a.answerSets[0].submissionDate});
      index++;
    });
    return data;
  }

  private getPaperAnswers(data): void {
    data.forEach(e => this.answers.push(e));
  }


  private getPaperQuestions(data): void {
    data.forEach(e => this.questions.push(e));
  }

  public back() {
    this.router.navigate(['/paper-view']);
    this.ngOnDestroy();
  }

  public goToResponse(data): void {
    this.currentQuestion = undefined;
    this.questionAnswerPairs = [];
    let set = new AnswerSetDao([]);
    this.answers.forEach(a => {
      if (data.username === a.answerSets[0].username) {
        set = a;
      }
    });
    this.generatePairs(set);
  }

  public getIndex(username: String) {
    let index = 1;
    this.answers.forEach(a => {
      if (username === a.answerSets[0].username) {
        return index;
      }
      index++;
    });
    return index;
  }

  public setView() {
    this.questionAnswerPairs = [];
    this.answers.forEach(set => {
      set.answerSets.forEach(answer => {
        if (this.currentQuestion.questionId === answer.questionId) {
          this.questionAnswerPairs.push(new QuestionAnswerPairDao(this.currentQuestion, answer));
        }
      });
    });
  }

}
